//=======================================================================
//                        MicroBoard LX9 spiMaster
//
// File:
//    fm_mbbridge.v
//
// Abstract:
//    Microblaze MCS IO Bus bridge
//
// Created:
//    4 December 2013
//
//======================================================================
//
// Copyright (c) 2013, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//  Revision History
// $Date: 2013-12-06 16:31:37 +0900 (Fri, 06 Dec 2013) $
// $Rev: 11 $

module fm_mbbridge (
  clk_core,
  rst_x,
  // microblaze side
  IO_Addr_Strobe,
  IO_Read_Strobe,
  IO_Write_Strobe,
  IO_Ready,
  IO_Read_Data,
  IO_Address,
  IO_Byte_Enable,
  IO_Write_Data,
  // internal side
  o_req,
  o_wr,
  o_adrs,
  i_ack,
  o_be,
  o_wd,
  i_rstr,
  i_rd
);

  input clk_core;
  input rst_x;
  // microblaze side
  input IO_Addr_Strobe;
  input IO_Read_Strobe;
  input IO_Write_Strobe;
  output IO_Ready;
  output [31:0] IO_Read_Data;
  input [31:0] IO_Address;
  input [3:0] IO_Byte_Enable;
  input [31:0] IO_Write_Data;
  // internal side
  output o_req;
  output o_wr;
  output [31:0] o_adrs;
  input i_ack;
  output [3:0] o_be;
  output [31:0] o_wd;
  input i_rstr;
  input [31:0] i_rd;
   
  reg r_IO_Addr_Strobe;
  reg r_IO_Write_Strobe;
  reg [31:0] r_IO_Address;
  reg [3:0]  r_IO_Byte_Enable;
  reg [31:0] r_IO_Write_Data;

  // internal side
  assign o_req = r_IO_Addr_Strobe;
  assign o_wr = r_IO_Write_Strobe;
  assign o_adrs = r_IO_Address;
  assign IO_Ready = (r_IO_Write_Strobe) ? (r_IO_Addr_Strobe & i_ack):
                                          i_rstr;
  assign IO_Read_Data = i_rd;
  assign o_be = r_IO_Byte_Enable;
  assign o_wd = r_IO_Write_Data;

   always @(posedge clk_core or negedge rst_x) begin
     if (~rst_x) begin
       r_IO_Addr_Strobe <= 'd0;
       r_IO_Write_Strobe <= 'd0;
       r_IO_Address <= 'd0;
       r_IO_Byte_Enable <= 'd0;
       r_IO_Write_Data <= 'd0;
     end else begin
       if (IO_Addr_Strobe) begin
         r_IO_Addr_Strobe <= 1'b1;
         r_IO_Write_Strobe <= IO_Write_Strobe;
         r_IO_Address <= IO_Address;
         r_IO_Byte_Enable <= IO_Byte_Enable;
         r_IO_Write_Data <= IO_Write_Data;
       end else if (i_ack) begin
         r_IO_Addr_Strobe <= 1'b0;
       end
     end
   end

   
endmodule

   