//=======================================================================
//                        MicroBoard LX9 spiMaster
//
// File:
//    mb_top.v
//
// Abstract:
//    OpenCores spiMaster implementation top module
//
// Created:
//    4 December 2013
//
//======================================================================
//
// Copyright (c) 2013, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//  Revision History
// $Date: 2013-12-06 16:31:37 +0900 (Fri, 06 Dec 2013) $
// $Rev: 11 $

module mb_top
  (
  input  CLK_100MHZ,
  input  CLK_66MHZ,
  input  CLK_40MHZ,
  input  USER_RESET,
  output [3:0] LED,
  input UART_Rx,
  output UART_Tx,
  // SDCARD
  output SD_CS,
  output SD_DO,
  input  SD_DI,
  output SD_CLK
  );

  //******************************************************************//
  // Create clock.                                                    //
  //******************************************************************//

  wire  clk_core;
  wire  clk20;
  wire  clk_enable;
  wire  pll_locked;
  wire  rst_x;
  wire  rst;
// microblaze_mcs
  wire w_IO_Read_Strobe;
  wire w_IO_Addr_Strobe;
  wire w_IO_Ready;
  wire w_IO_Write_Strobe;
  wire [31:0] w_IO_Address;
  wire [3:0]  w_IO_Byte_Enable;
  wire [31:0] w_IO_Write_Data;
  wire [31:0] w_IO_Read_Data;

// bridge
  wire        w_req;
  wire        w_wr;
  wire [31:0] w_adrs;
  wire        w_ack;
  wire [3:0]  w_be;
  wire [31:0] w_wd;
  wire        w_rstr;
  wire [31:0] w_rd;

  wire w_FIT1_Toggle;

  assign rst_x = ~rst;
  assign w_rstr = (w_req & !w_wr & w_ack);
  // 32bit - 8bit converson
  wire [7:0] w_adrs_b;
  wire [7:0] w_wd_b;
  wire [7:0] w_rd_b;
  assign w_rd = {'d4{w_rd_b}};
  assign w_ba = w_be[1] ? 2'd1 :
                w_be[2] ? 2'd2 : 
                w_be[3] ? 2'd3 : 
                          2'd0 ;
  assign w_adrs_b = w_adrs[7:0];
  assign w_wd_b = (w_ba == 'd1) ? w_wd[15:8]:
                  (w_ba == 'd2) ? w_wd[23:16]:
                  (w_ba == 'd3) ? w_wd[31:24]:
                                  w_wd[7:0];

clk_wiz_v3_6 clk_wiz (
  .CLK_IN1(CLK_100MHZ),
  // Clock out ports
  .CLK_OUT1(clk_core),
  .CLK_OUT2(clk20),
  // Status and control signals
  .RESET(USER_RESET),
  .LOCKED(pll_locked)
);
   
microblaze_mcs_v1_4 mcs_0 (
  .Clk(clk_core),
  .Reset(~rst_x),
  .UART_Tx(UART_Tx),
  .UART_Rx(UART_Rx),
  .GPI1_Interrupt(),
  .GPI1({4'ha,
         w_c3_p0_rd_empty,
         w_c3_p0_cmd_full,
         /*w_c3_p0_wr_full,
           w_c3_calib_done*/
                          w_vint_x,
                          w_int
                          }),
  .FIT1_Interrupt(),
  .FIT1_Toggle(w_FIT1_Toggle),
  .INTC_Interrupt(w_int),
  .FIT2_Interrupt(),
  .FIT2_Toggle(w_FIT1_Toggle),
  .IO_Addr_Strobe(w_IO_Addr_Strobe),
  .IO_Read_Strobe(w_IO_Read_Strobe),
  .IO_Write_Strobe(w_IO_Write_Strobe),
  .IO_Ready(w_IO_Ready),
  .IO_Read_Data(w_IO_Read_Data),
  .IO_Address(w_IO_Address),
  .IO_Byte_Enable(w_IO_Byte_Enable),
  .IO_Write_Data(w_IO_Write_Data)
);
  
fm_mbbridge u_mbbridge (
  .clk_core(clk_core),
  .rst_x(rst_x),
  // microblaze side
  .IO_Addr_Strobe(w_IO_Addr_Strobe),
  .IO_Read_Strobe(w_IO_Read_Strobe),
  .IO_Write_Strobe(w_IO_Write_Strobe),
  .IO_Ready(w_IO_Ready),
  .IO_Read_Data(w_IO_Read_Data),
  .IO_Address(w_IO_Address),
  .IO_Byte_Enable(w_IO_Byte_Enable),
  .IO_Write_Data(w_IO_Write_Data),
  // internal side
  .o_req(w_req),
  .o_wr(w_wr),
  .o_adrs(w_adrs),
  .i_ack(w_ack),
  .o_be(w_be),
  .o_wd(w_wd),
  .i_rstr(w_rstr),
  .i_rd(w_rd)
);

spiMaster u_spiMaster (
  .clk_i(clk_core),
  .rst_i(rst),
  .address_i(w_adrs_b),
  .data_i(w_wd_b),
  .data_o(w_rd_b),
  .strobe_i(w_req),
  .we_i(w_wr),
  .ack_o(w_ack),

  // SPI logic clock
  .spiSysClk(clk20),

  //SPI bus
  .spiClkOut(SD_CLK),
  .spiDataIn(SD_DI),
  .spiDataOut(SD_DO),
  .spiCS_n(SD_CS)
);

//******************************************************************//
//                                                                  //
//******************************************************************//

  reg  [26:0] led_count;
  reg  r_sd_do;
  always @(posedge clk_core) 
        led_count <= led_count + 1;
  
  always @(posedge clk20) 
    r_sd_do <= SD_DO;

  assign LED[3:2] = led_count[26:25];
  assign LED[0] = pll_locked;
  assign LED[1] = r_sd_do;
  
endmodule
