/*-----------------------------------------------------------------------
/  Low level disk interface modlue include file   (C)ChaN, 2009
/-----------------------------------------------------------------------*/
// MicroBlaze porting K.I

#ifndef _DISKIO
#define _DISKIO

#define SPIMASTER_BASE 0xc0000000
#define SPI_MASTER_VERSION (*(volatile unsigned char *)(SPIMASTER_BASE + 0x0))
#define SPI_MASTER_CONTROL (*(volatile unsigned char *)(SPIMASTER_BASE + 0x1))
#define SPI_TRANS_TYPE (*(volatile unsigned char *)(SPIMASTER_BASE + 0x2))
#define SPI_TRANS_CTRL (*(volatile unsigned char *)(SPIMASTER_BASE + 0x3))
#define SPI_TRANS_STS (*(volatile unsigned char *)(SPIMASTER_BASE + 0x4))
#define SPI_TRANS_ERROR (*(volatile unsigned char *)(SPIMASTER_BASE + 0x5))
#define SPI_DIRECT_ACCESS_DATA (*(volatile unsigned char *)(SPIMASTER_BASE + 0x6))
#define SPI_SD_ADDR_7_0 (*(volatile unsigned char *)(SPIMASTER_BASE + 0x7))
#define SPI_SD_ADDR_15_8 (*(volatile unsigned char *)(SPIMASTER_BASE + 0x8))
#define SPI_SD_ADDR_23_16 (*(volatile unsigned char *)(SPIMASTER_BASE + 0x9))
#define SPI_SD_ADDR_31_24 (*(volatile unsigned char *)(SPIMASTER_BASE + 0xa))
#define SPI_CLK_DEL (*(volatile unsigned char *)(SPIMASTER_BASE + 0xb))
#define SPI_RX_FIFO_DATA (*(volatile unsigned char *)(SPIMASTER_BASE + 0x10))
#define SPI_RX_FIFO_DATA_COUNT_MSB (*(volatile unsigned char *)(SPIMASTER_BASE + 0x12))
#define SPI_RX_FIFO_DATA_COUNT_LSB (*(volatile unsigned char *)(SPIMASTER_BASE + 0x13))
#define SPI_RX_FIFO_CONTROL (*(volatile unsigned char *)(SPIMASTER_BASE + 0x14))
#define SPI_TX_FIFO_DATA (*(volatile unsigned char *)(SPIMASTER_BASE + 0x20))
#define SPI_TX_FIFO_CONTROL (*(volatile unsigned char *)(SPIMASTER_BASE + 0x24))

#define _USE_WRITE      1       /* 1: Enable write function */
#define _USE_IOCTL      1       /* 1: Enable ioctl function */

#include "integer.h"


/* Status of Disk Functions */
typedef BYTE    DSTATUS;

/* Results of Disk Functions */
typedef enum {
        RES_OK = 0,             /* 0: Successful */
        RES_ERROR,              /* 1: R/W Error */
        RES_WRPRT,              /* 2: Write Protected */
        RES_NOTRDY,             /* 3: Not Ready */
        RES_PARERR              /* 4: Invalid Parameter */
} DRESULT;


/*---------------------------------------*/
/* Prototypes for disk control functions */

DSTATUS disk_initialize (BYTE pdrv);
DSTATUS disk_status (BYTE pdrv);
DRESULT disk_read (BYTE pdrv, BYTE* buff, DWORD sector, BYTE count);
#if     _USE_WRITE
DRESULT disk_write (BYTE pdrv, const BYTE* buff, DWORD sector, BYTE count);
#endif
#if     _USE_IOCTL
DRESULT disk_ioctl (BYTE pdrv, BYTE cmd, void* buff);
#endif
void disk_timerproc (void);


/* Disk Status Bits (DSTATUS) */

#define STA_NOINIT              0x01    /* Drive not initialized */
#define STA_NODISK              0x02    /* No medium in the drive */
#define STA_PROTECT             0x04    /* Write protected */


/* Command code for disk_ioctrl() */

/* Generic ioctl command (defined for FatFs) */
#define CTRL_SYNC                       0       /* Flush disk cache (for write functions) */
#define GET_SECTOR_COUNT        1       /* Get media size (for only f_mkfs()) */
#define GET_SECTOR_SIZE         2       /* Get sector size (for multiple sector size (_MAX_SS >= 1024)) */
#define GET_BLOCK_SIZE          3       /* Get erase block size (for only f_mkfs()) */
#define CTRL_ERASE_SECTOR       4       /* Force erased a sector group (for only _USE_ERASE) */

/* Generic ioctl command */
#define CTRL_POWER                      5       /* Get/Set power status */
#define CTRL_LOCK                       6       /* Lock/Unlock media removal */
#define CTRL_EJECT                      7       /* Eject media */

/* MMC/SDC specific ioctl command */
#define MMC_GET_TYPE            10      /* Get card type */
#define MMC_GET_CSD                     11      /* Get CSD */
#define MMC_GET_CID                     12      /* Get CID */
#define MMC_GET_OCR                     13      /* Get OCR */
#define MMC_GET_SDSTAT          14      /* Get SD status */

/* ATA/CF specific ioctl command */
#define ATA_GET_REV                     20      /* Get F/W revision */
#define ATA_GET_MODEL           21      /* Get model name */
#define ATA_GET_SN                      22      /* Get serial number */

/* NAND specific ioctl command */
#define NAND_FORMAT                     30      /* Create physical format */



/* Card type flags (CardType) */

#define CT_MMC                          0x01
#define CT_SD1                          0x02
#define CT_SD2                          0x04
#define CT_SDC                          (CT_SD1|CT_SD2)
#define CT_BLOCK                        0x08

#endif
