/*------------------------------------------------------------------------/
/  Universal string handler for user console interface
/-------------------------------------------------------------------------/
/
/  Copyright (C) 2011, ChaN, all right reserved.
/
/ * This software is a free software and there is NO WARRANTY.
/ * No restriction on use. You can use, modify and redistribute it for
/   personal, non-profit or commercial products UNDER YOUR RESPONSIBILITY.
/ * Redistributions of source code must retain the above copyright notice.
/
/-------------------------------------------------------------------------*/
// MicroBlaze porting K.I

#include "xprintf.h"


#if _USE_XFUNC_OUT
#include <stdarg.h>
void (*xfunc_out)(unsigned char);       /* Pointer to the output stream */
static char *outptr;

/*----------------------------------------------*/
/* Put a character                              */
/*----------------------------------------------*/

void xputc (char c)
{
putchar(c);
}



/*----------------------------------------------*/
/* Put a null-terminated string                 */
/*----------------------------------------------*/

void xputs (                                    /* Put a string to the default device */
        const char* str                         /* Pointer to the string */
)
{
        while (*str)
                xputc(*str++);
}



/*----------------------------------------------*/
/* Dump a line of binary dump                   */
/*----------------------------------------------*/

void put_dump (
        const void* buff,               /* Pointer to the array to be dumped */
        unsigned long addr,             /* Heading address value */
        int len,                        /* Number of items to be dumped */
        int width                       /* Size of the items (DW_CHAR, DW_SHORT, DW_LONG) */
)
{
        int i;
        const unsigned char *bp;
        const unsigned short *sp;
        const unsigned long *lp;


        xil_printf("%08lX:", addr);             /* address */

        switch (width) {
        case DW_CHAR:
                bp = buff;
                for (i = 0; i < len; i++)               /* Hexdecimal dump */
                        xil_printf(" %02X", bp[i]);
                xputc(' ');
                for (i = 0; i < len; i++)               /* ASCII dump */
                        xputc((bp[i] >= ' ' && bp[i] <= '~') ? bp[i] : '.');
                break;
        case DW_SHORT:
                sp = buff;
                do                                                              /* Hexdecimal dump */
                        xil_printf(" %04X", *sp++);
                while (--len);
                break;
        case DW_LONG:
                lp = buff;
                do                                                              /* Hexdecimal dump */
                        xil_printf(" %08LX", *lp++);
                while (--len);
                break;
        }

        xputc('\n');
}

#endif /* _USE_XFUNC_OUT */



#if _USE_XFUNC_IN
unsigned char (*xfunc_in)(void);        /* Pointer to the input stream */

/*----------------------------------------------*/
/* Get a line from the input                    */
/*----------------------------------------------*/

int xgets (             /* 0:End of stream, 1:A line arrived */
        char* buff,     /* Pointer to the buffer */
        int len         /* Buffer length */
)
{
        int c, i;

        if (!xfunc_in) return 0;                /* No input function specified */

        i = 0;
        for (;;) {
                c = xfunc_in();                 /* Get a char from the incoming stream */
                if (!c) continue;               /* End of stream? */
                //if (!c) return 0;             /* End of stream? */
                if (c == '\r') break;           /* End of line? */
                if (c == '\b' && i) {           /* Back space? */
                        i--;
                        if (_LINE_ECHO) xputc(c);
                        continue;
                }
                if (c >= ' ' && i < len - 1) {  /* Visible chars */
                        buff[i++] = c;
                        if (_LINE_ECHO) xputc(c);
                }
        }
        buff[i] = 0;    /* Terminate with a \0 */
        if (_LINE_ECHO) xputc('\n');
        return 1;
}



/*----------------------------------------------*/
/* Get a value of the string                    */
/*----------------------------------------------*/
/*      "123 -5   0x3ff 0b1111 0377  w "
            ^                           1st call returns 123 and next ptr
               ^                        2nd call returns -5 and next ptr
                   ^                3rd call returns 1023 and next ptr
                          ^         4th call returns 15 and next ptr
                               ^    5th call returns 255 and next ptr
                                  ^ 6th call fails and returns 0
*/

int xatoi (                     /* 0:Failed, 1:Successful */
        char **str,             /* Pointer to pointer to the string */
        long *res               /* Pointer to the valiable to store the value */
)
{
        unsigned long val;
        unsigned char c, r, s = 0;


        *res = 0;

        while ((c = **str) == ' ') (*str)++;    /* Skip leading spaces */

        if (c == '-') {         /* negative? */
                s = 1;
                c = *(++(*str));
        }

        if (c == '0') {
                c = *(++(*str));
                switch (c) {
                case 'x':               /* hexdecimal */
                        r = 16; c = *(++(*str));
                        break;
                case 'b':               /* binary */
                        r = 2; c = *(++(*str));
                        break;
                default:
                        if (c <= ' ') return 1; /* single zero */
                        if (c < '0' || c > '9') return 0;       /* invalid char */
                        r = 8;          /* octal */
                }
        } else {
                if (c < '0' || c > '9') return 0;       /* EOL or invalid char */
                r = 10;                 /* decimal */
        }

        val = 0;
        while (c > ' ') {
                if (c >= 'a') c -= 0x20;
                c -= '0';
                if (c >= 17) {
                        c -= 7;
                        if (c <= 9) return 0;   /* invalid char */
                }
                if (c >= r) return 0;           /* invalid char for current radix */
                val = val * r + c;
                c = *(++(*str));
        }
        if (s) val = 0 - val;                   /* apply sign if needed */

        *res = val;
        return 1;
}

#endif /* _USE_XFUNC_IN */
